@@ -2,7 +2,44 @@ import copy
 import socket
 import concurrent.futures
 import yaml
-from netaddr import IPAddress, IPNetwork, IPSet, AddrFormatError
+from netaddr import IPAddress, IPSet, AddrFormatError
+
+IPV4_RESERVED_NETWORK_LIST = [
+    "0.0.0.0/8",
+    "100.64.0.0/10",
+    "127.0.0.0/8",
+    "169.254.0.0/16",
+    "192.0.0.0/24",
+    "192.0.2.0/24",
+    "192.88.99.0/24",
+    "198.51.100.0/24",
+    "203.0.113.0/24",
+    "224.0.0.0/4",
+    "233.252.0.0/24",
+    "240.0.0.0/4",
+    "255.255.255.255/32",
+]
+
+IPV4_RFC1918_NETWORK_LIST = [
+    "10.0.0.0/8",
+    "172.16.0.0/12",
+    "192.168.0.0/16",
+]
+
+IPV6_RESERVED_NETWORK_LIST = [
+    "::ffff:0:0/96",
+    "::ffff:0:0:0/96",
+    "64:ff9b::/96",
+    "64:ff9b:1::/48",
+    "100::/64",
+    "2001:0000::/32",
+    "2001:20::/28",
+    "2001:db8::/32",
+    "2002::/16",
+    "fc00::/7",
+    "fe80::/10",
+    "ff00::/8",
+]
 
 
 class rdns_reaper:
@@ -11,12 +48,9 @@ class rdns_reaper:
         self._dns_dict = dict()
         self._resolver_ip = None
 
-        if "limit_to_rfc1918" in kwargs.keys():
-            if isinstance(kwargs["limit_to_rfc1918"], bool):
-                self._limit_to_rfc1918 = kwargs["limit_to_rfc1918"]
-            else:
-                # User passed an invalid response
-                self._limit_to_rfc1918 = False
+        """Check for RFC1918 filtering"""
+        if "limit_to_rfc1918" in kwargs:
+            self.limit_to_rfc1918(kwargs["limit_to_rfc1918"])
         else:
             self._limit_to_rfc1918 = False
 
@@ -27,7 +61,7 @@ class rdns_reaper:
             else:
                 raise TypeError
         except KeyError:
-            self._concurrent = 1
+            self._concurrent = 5
 
         """Determine how to mark unresolvable entries"""
         try:
@@ -197,6 +231,7 @@ class rdns_reaper:
         """Set the RFC1918 filter."""
         if type(value) is not bool:
             raise TypeError
+        self._limit_to_rfc1918 = value
 
     def loadfile(self, filename):
         """Load saved data in YAML format."""
@@ -215,27 +250,38 @@ class rdns_reaper:
             return False
         return True
 
+    def _build_resolve_list(self):
+        """Build list of IP's to perform resolver on, shared by serial and parallel methods."""
+        initial_pending_ips = [
+            key for key, value in self._dns_dict.items() if value is None
+        ]
+
+        if self._limit_to_rfc1918:
+            IPv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
+        else:
+            IPv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
+
+        IPv6_skipped_networks = IPSet(IPV6_RESERVED_NETWORK_LIST)
+
+        pending_ips = list()
+
+        for key in initial_pending_ips:
+            address = IPAddress(key)
+
+            if self._limit_to_rfc1918 is False:
+                if (address.version == 4) and (address not in IPv4_skipped_networks):
+                    pending_ips.append(key)
+                elif (address.version == 6) and (address not in IPv6_skipped_networks):
+                    pending_ips.append(key)
+            else:
+                if address in IPSet(IPV4_RFC1918_NETWORK_LIST):
+                    pending_ips.append(key)
+
+        return pending_ips
+
     def resolve_all(self):
         """Resolve all unknown IPs in parallel."""
-        """TODO: Look at doing this in a less resource intensive way"""
-        if self._limit_to_rfc1918:
-            pending_ips = [
-                key
-                for key, value in self._dns_dict.items()
-                if value is None
-                and IPAddress(key).version == 4
-                and self._isreservedIPv4(key) is False
-                and self._isrfc1918(key) is True
-            ]
-        else:
-            pending_ips = [
-                key
-                for key, value in self._dns_dict.items()
-                if value is None
-                and IPAddress(key).version == 4
-                and self._isreservedIPv4(key) is False
-            ]
-
+        pending_ips = self._build_resolve_list()
         with concurrent.futures.ThreadPoolExecutor(
             max_workers=self._concurrent
         ) as executor:
@@ -243,20 +289,10 @@ class rdns_reaper:
 
     def resolve_all_serial(self):
         """Resolve all unknown IPs serially."""
-        pending_ips = [key for key, value in self._dns_dict.items() if value is None]
+        pending_ips = self._build_resolve_list()
 
-        for key in pending_ips:
-            if IPAddress(key).version != 4:
-                raise TypeError("Only IPv4 addresses are accepted at this time")
-
-            if self._limit_to_rfc1918:
-                if (self._isrfc1918(key) is False) and (
-                    self._isreservedIPv4(key) is False
-                ):
-                    self._resolve_function(key)
-            else:
-                if self._isreservedIPv4(key) is False:
-                    self._resolve_function(key)
+        for address in pending_ips:
+            self._resolve_function(address)
 
     def savefile(self, filename=None):
         """Save internal dictionary to YAML file."""
@@ -298,32 +334,38 @@ class rdns_reaper:
 
         if address in rfc1918_IPSet:
             return True
-
         return False
 
     @staticmethod
     def _isreservedIPv4(address_txt):
         """Determine if an address is reserved (loopbacks, documentation, etc) or not."""
         address = IPAddress(address_txt)
-        reserved_network_list = [
-            "0.0.0.0/8",
-            "100.64.0.0/10",
-            "127.0.0.0/8",
-            "169.254.0.0/16",
-            "192.0.0.0/24",
-            "192.0.2.0/24",
-            "192.88.99.0/24",
-            "198.51.100.0/24",
-            "203.0.113.0/24",
-            "224.0.0.0/4",
-            "233.252.0.0/24",
-            "240.0.0.0/4",
-            "255.255.255.255/32",
-        ]
-        reserved_network_IPSet = IPSet(reserved_network_list)
-        if address_txt in reserved_network_IPSet:
-            return True
-        return False
+
+        if address.version == 4:
+            reserved_network_IPSet = IPSet(IPV4_RESERVED_NETWORK_LIST)
+            if address_txt in reserved_network_IPSet:
+                return True
+            return False
+        elif address.version == 6:
+            reserved_network_IPSet = IPSet(IPV6_RESERVED_NETWORK_LIST)
+            if address_txt in reserved_network_IPSet:
+                return True
+            return False
+        else:
+            raise ValueError
+
+    @staticmethod
+    def _isreservedIPv6(address_txt):
+        """Determine if an address is reserved (loopbacks, documentation, etc) or not."""
+        address = IPAddress(address_txt)
+
+        if address.version == 6:
+            reserved_network_IPSet = IPSet(IPV6_RESERVED_NETWORK_LIST)
+            if address_txt in reserved_network_IPSet:
+                return True
+            return False
+        else:
+            raise ValueError
 
     class _reaper_iterator:
         def __init__(self, parentclass):
